import sys

sys.setrecursionlimit(2000000000)

print ("Will evaluate three methods of calculating 35th element in fibonacci sequence. \n")

#define a simple recursive fibonacci sequence. 
def recursive_f(n):
    """Slow recursive function. n = the number to evaluate. """
    
    if n == 0:
        return 0     #base case
    elif n == 1:
        return 1     #another base case
    else:
        return recursive_f(n-1)+recursive_f(n-2)


#same as the previous function, but store previous values.
#this keeps the function from re-evaluating the same values. 
def mem_f(n, mem=dict()):
    """Memoization recursive method. n = num to eval. Don't pass mem. """
    
    if (n in mem):
        return mem[n]   #welp we already saved a value. Don't re-evaluate. 
    
    if n == 0:
        return 0    #base case  
    elif n == 1:
        return 1    #base case
    else:
        value = mem_f(n-1, mem)+ mem_f(n-2, mem)
        mem[n] = value  #store value
        return value


#This solution is a little different. This is a formula that is not recursive.
#In theory python shouldn't crash, but large numbers are beyond python's range.     
def fib_exact(n):
    """Calculated exact solution from second order sequence. n = num to eval."""
    
    value1 = ((1 + (5**(0.5)))**n)
    value2 = ((1 - (5**(0.5)))**n)
    value3 = (2**n) * (5**(0.5))

    result = (value1 - value2) // value3
    return int(result) 


#recursive with slow method. will take forever.
print ("Slow recursive method: ", end="")
print (str(recursive_f(35)) + "\n")

#recursive with fast method. Much faster.
print ("Fast recursive method: ", end="") 
print(str(mem_f(35)) + "\n")

#Exact solution. The fastest of them all.
print ("Exact non-recursive method: ", end="") 
print (str(fib_exact(35))) 
